#include <iomanip>
#include <string>
#include <iostream>

namespace {

  long double y = 0.0;            // years                   -- user supplied
  long double u = 0.0;            // periods/year            -- user supplied
  long long   n = 0;              // Total number of periods -- calculated
  long double i = 0.0;            // Annual interest rate    -- user supplied
  long double r = 0.0;            // Rate per period         -- calculated

  long double k = 0.0;            // Yearly contribution     -- user supplied
  long double c = 0.0;            // contribution/period     -- calculated

  long double total_c = 0.0;      // Your total contribution, i.e., how
                                  // much you invested.      -- calculated

  long double p = 0.0;            // Principle, i.e., your grand total.
                                  //                         -- calculated

  bool is_y_ready = false;
  bool is_u_ready = false;
  bool is_i_ready = false;
  bool is_k_ready = false;

  std::string progname("systematic_investing");

  long double cent_precision(long double x)
  {
    return static_cast<long long>(x * 100)/100.0;
  }

  std::string basename(const std::string& fullpath)
  {
    return std::string(fullpath.rfind("/"), std::string::npos);
  }

  std::string usage()
  {
    std::string tmp;
    tmp += "Usage: " + progname + " [<flags>]\n";
    tmp += "\n";
    tmp += "       flags:  All flags are required.\n";
    tmp += "\n";
    tmp += "           -i : Annual interest rate.  For example, if you\n";
    tmp += "                expect to earn 12.48%, use \"-i .1248\"\n";
    tmp += "           -k : Yearly contribution.  It will be invested evenly\n";
    tmp += "                throughout the year in each period described by\n";
    tmp += "                the -u option.\n";
    tmp += "           -u : Periods/Year.  For example: If you invest every\n";
    tmp += "                two weeks, use \"-u 26\".\n";
    tmp += "           -y : Years.\n";
    tmp += "\n";
    return tmp;
  }

  void verify_parameters()
  {
    if( !is_i_ready ) {
      std::cerr << "\n" << progname
                << ": Error: You need to supply the annual interest rate.\n\n";
      std::cerr << usage() << std::endl;
      exit(1);
    }
    if( !is_k_ready ) {
      std::cerr << "\n" << progname
                << ": Error: You need to supply the yearly contribution.\n\n";
      std::cerr << usage() << std::endl;
      exit(1);
    }
    if( !is_u_ready ) {
      std::cerr << "\n" << progname
                << ": Error: You need to supply the number of periods/year.\n"
                << "\n";
      std::cerr << usage() << std::endl;
      exit(1);
    }
    if( !is_y_ready ) {
      std::cerr << "\n" << progname
                << ": Error: You need to supply the number of years.\n\n";
      std::cerr << usage() << std::endl;
      exit(1);
    }
  }

  void set_parameters(int argc, char** argv)
  {

    for( int j = 1 ; j < argc ; ++j ) {
      std::string arg(argv[j]);

      // Annual interest rate.
      if( (arg == "-i") && (j+1 < argc) ) {
        arg = argv[++j];
        i = atof(arg.c_str());
        is_i_ready = true;
      }

      // Yearly contribution.
      if( (arg == "-k") && (j+1 < argc) ) {
        arg = argv[++j];
        k = atof(arg.c_str());
        is_k_ready = true;
      }

      // Number of periods/year.
      if( (arg == "-u") && (j+1 < argc) ) {
        arg = argv[++j];
        long long tmp = atol(arg.c_str());
        if( tmp == 0 ) {
          std::cerr << "\n" << progname
                    << ": Error: The number of periods/year cannot be zero.\n"
                    << "\n";
          std::cerr << usage() << std::endl;
          exit(1);
        }
        u = tmp;
        is_u_ready = true;
      }

      // Number of years.
      if( (arg == "-y") && (j+1 < argc) ) {
        arg = argv[++j];
        y = atof(arg.c_str());
        is_y_ready = true;
      }
    }

    verify_parameters();

    //
    // Calculate some other values from the parameters the user gave.
    //

    // Calculate the contribution per period.  You need to truncate
    // anything smaller than a penny.
    c = cent_precision(k/u);
 
    // Calculate the effective rate per period.
    r = i/u;

    // Calculate the total number of periods for the entire calculation.
    n = static_cast<long long>(u * y);

  }

}

int main(int argc, char* argv[])
{
  int rv = 0;

  progname = basename(argv[0]);

  set_parameters(argc, argv);

  std::cout << std::endl;
  std::cout << "    Annual Interest Rate: " << i << std::endl;
  std::cout << "     Yearly Contribution: " << k << std::endl;
  std::cout << "            Periods/Year: " << u << std::endl;
  std::cout << "                   Years: " << y << std::endl;
  std::cout << std::endl;

  for( int j = 0 ; j < n ; ++j ) {
    total_c += c;
    p = ( p * (1 + r) ) + c;
    int tmp = j % static_cast<long long>(u) + 1;
    std::cout << std::setw(2) << j/static_cast<long long>(u) + 1 << "-"
              << ((tmp < 10) ? "0" : "") << tmp
              << ": Investing $" << total_c << " yields $"
              << std::fixed << std::setprecision(2) << cent_precision(p)
              << std::endl;
  }

  return rv;
}
